import getUrlContents from '../cdn/getUrlContents';
import { Product } from '../interfaces/ISettings';
import { SsnDiffType } from '../interfaces/ISsnFileEntry';
import extractFile from './extractFile';
import getSolidpkg from './getSolidpkg';
import readSsnFile from './readSsnFile';

export default async function getPatch(product: Product, from: number, to: number) {
  const solidPkg = await getSolidpkg(product, from, to);

  const bufferArray = await Promise.all(solidPkg.files.map((file) => getUrlContents({ host: 'cdn-patch.swtor.com', path: `/patch/${product}/${product}_${from}to${to}/${file.name}` })));
  const zipFile = bufferArray[bufferArray.length - 1];
  const dvArray = bufferArray.map((buffer) => new DataView(buffer));

  const fileEntries = readSsnFile(zipFile);

  //Verify file entries
  if (from === -1) {
    fileEntries.filter((file) => file.diffType !== SsnDiffType.NewFile).forEach((file) => {
      throw new Error(`Patches from -1 must be included in full, but this patch had a file "${file.name}" with diff type ${file.diffType}.`);
    });
  }
  //TODO: last file must always be `${product}.version` with diff type. Other files depend on diffType.
  console.log(fileEntries);

  //Extract newly added files
  fileEntries.filter((file) => file.diffType === SsnDiffType.NewFile).forEach(async (file) => {
    const fileContents = await extractFile(file, dvArray);
    console.log(fileContents);
    //TODO
  });

  //Extract changed files
  fileEntries.filter((file) => file.diffType === SsnDiffType.Changed).forEach(async (file) => {
    const fileContents = await extractFile(file, dvArray);
    console.log(fileContents);
    //TODO
  });

  //Need to delete deleted files
  fileEntries.filter((file) => file.diffType === SsnDiffType.Deleted).forEach((file) => {
    //TODO
  });
}
