import ISolid from '../interfaces/ISolidFile';

/** Verifies metafile.solid for correctness. */
export default function verifySolidpkg(file: ISolid, { product, from, to }: {product: string, from: number, to: number}) {
  if (typeof file['creation date'] !== 'number') {
    throw new Error(`Expected creation date to be a number but it was "${file['creation date']}".`);
  }
  if (file.announce.match(/^http:\/\/Tracker22\.[0-9a-z]{8}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{12}\.automated\.(tel\.swtor\.com|ssntracker\.int):80\/$/) === null) {
    throw new Error(`Expected announce URL but it was "${file.announce}".`);
  }
  if (file.title !== `${product}: ${from}to${to}`) {
    throw new Error(`Expected title "${product}: ${from}to${to}" but it was "${file.title}".`);
  }
  if (file.networkgroupid !== 16097) {
    throw new Error(`Expected networkgroupid 16097 but it was "${file.networkgroupid}".`);
  }
  if (file.reliable !== `http://cdn-patch.swtor.com/patch/${product}/${product}_${from}to${to}/`) {
    throw new Error(`Expected reliable URL but it was "${file.reliable}".`);
  }
  if (file['reliable-id'] !== '0') {
    throw new Error(`Expected reliable-id to be "0" but it was "${file['reliable-id']}".`);
  }

  if (file.info === undefined) {
    throw new Error(`Expected info field but it was missing.`);
  }

  if (!Array.isArray(file.info.files)) {
    throw new Error(`Expected files field to be an array but it isn't.`);
  }
  for (let i = 0, il = file.info.files.length; i < il; i += 1) {
    const fileEntry = file.info.files[i];
    if (typeof fileEntry.length !== 'number' && fileEntry.length >= 0 && fileEntry.length <= 1700000000) {
      throw new Error(`Expected file length to be a number but it was ${fileEntry.length}.`);
    }
    if (!Array.isArray(fileEntry.path) || fileEntry.path.length !== 1) {
      throw new Error(`Expected valid file name but it was not an array with one element.`);
    }
    if (typeof fileEntry.path[0] !== 'string' || !fileEntry.path[0].match(new RegExp(`${product}_${from}to${to}\.z(ip|0[1-9]|[1-9][0-9])$`))) {
      throw new Error(`Expected valid file name but it was ${fileEntry.path[0]}.`);
    }
  }

  if (file.info['piece length'] !== 65536 && file.info['piece length'] !== 4194304 ) {
    throw new Error(`Expected piece length to be "65536" or "4194304" but it was "${file.info['piece length']}".`);
  }
  if (typeof file.info.pieces !== 'string') {
    throw new Error(`Expected pieces to be a string but it was "${file.info.pieces}".`);
  }
  if (file.info.private !== 0) {
    throw new Error(`Expected file to not be private ("0") but it was "${file.info.private}".`);
  }
  if (file.info.closed !== 1) {
    throw new Error(`Expected file to be closed ("1") but it was "${file.info.closed}".`);
  }
  if (typeof file.info.uniqueid !== 'string') {
    throw new Error(`Expected uniqueid to be a string but it was "${file.info.uniqueid}".`);
  }
}
